//
//  SCSCustomModifierView.swift
//  SciChartSwiftDemo
//
//  Created by Admin on 9/20/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import UIKit
import SciChart

class CustomModifier : SCIGestureModifier {
    // marker to be displayed
    var _marker : SCIEllipsePointMarker! = nil
    
    // position of tap gesture
    var _location : CGPoint = CGPoint.zero
    
    // if marker should be visible
    var _visible : Bool = false
    
    // x data value at point marker
    var _xValue : Double = 0
    
    // y data value at point marker
    var _yValue : Double = 0
    
    // index of data point at point marker position
    var _index : Int = 0
    
    // renderable series to whicj point marker is bound
    var _rSeries : SCIRenderableSeries?
    
    // control panel for additional actions with modifier
    weak var _controlPanel : CustomModifierControlPanel?
    
    
    init(controlPanel : CustomModifierControlPanel) {
        super.init()
        
        initializeControlPanel(controlPanel)
        initializePointMarker()
    }
    
    func initializeControlPanel(controlPanel : CustomModifierControlPanel){
        _controlPanel = controlPanel
        
        weak var wSelf = self
        _controlPanel?.onClearClicked = { () -> Void in wSelf!.hideMarker() }
        _controlPanel?.onNextClicked = { () -> Void in  wSelf!.moveMarker(+1) }
        _controlPanel?.onPrevClicked = { () -> Void in wSelf!.moveMarker(-1) }

    }
    
    func initializePointMarker(){
        _marker = SCIEllipsePointMarker()
        _marker.fillBrush = SCIBrushSolid(color: UIColor.redColor() )
        _marker.drawBorder = false;
    }
    
    func moveMarker (index: Int) {
        if (!_visible) {
            return
        }
        
        let context = parentSurface.renderSurface.modifierContext();
        let dataSeries : SCIDataSeriesProtocol = _rSeries!.currentRenderPassData.dataSeries()
        
        // check if index is out of data series range
        _index += index;
        let indexOutOfRange = _index < 0 || Int32(_index) >= dataSeries.count()
        
        if (indexOutOfRange) {
            _visible = false
            context.invalidate()
            return
        }
        
        // get data values from data series for new index
        _xValue = SCIGenericDouble(dataSeries.xValues().valueAt(_index))
        _yValue = SCIGenericDouble(dataSeries.yValues().valueAt(_index))
        
        if (isnan(_xValue) || isnan(_yValue)) {
            _visible = false
            context.invalidate()
            return;
        }
        // trigger modifier context to redraw
        context.invalidate()
    }
    
    func hideMarker () {
        _visible = false;
        // modifier context should be invalidated to trigger modifier redraw
        let parent : SCIChartSurfaceProtocol = self.parentSurface
        let context : SCIRenderContext2D = parent.renderSurface.modifierContext()
        context.invalidate()
    }
    
    func preapareDataForDrawing () {
        _visible = false
        
        let series = parentSurface.renderableSeries()
        
        // gesture coordinates is connected to whole surface, but calculators get coordinates related to chart surface only
        // we need to get location on chart surface
        let actualLocation : CGPoint = parentSurface.renderSurface.pointInChartFrame(_location)
        
        // check every renderable series for hit
        for  i in 0..<series.count {
            let rSeries = series[i] as! SCIRenderableSeries
            let data = rSeries.currentRenderPassData
            
            // get hit test tools
            let hitTest = rSeries.hitTestProvider()
            
            // hit test verticaly: check if vertical projection through touch location crosses chart
            let hitTestResult = hitTest.hitTestVerticalAtX(Double(actualLocation.x),
                                                                           y: Double(actualLocation.y),
                                                                           radius: 5,
                                                                           onData: data)
            
            if (hitTestResult.match) {
                // if hit is registered on series
                // get values at closest point to hit test position
                _xValue = SCIGenericDouble(hitTestResult.xValue)
                _yValue = SCIGenericDouble(hitTestResult.yValue)
                _index = Int(hitTestResult.index)
                
                if (isnan(_xValue) || isnan(_yValue)) {
                    continue
                }
                
                _visible = true
                _rSeries = rSeries // will be used during drawing
                break
            }
        }
    }
    
    override func onTapGesture (gesture: UITapGestureRecognizer!, at view: UIView!) -> Bool {
        
        let location = gesture.locationInView(view)
        let rs = self.parentSurface.renderSurface
        
        if (!rs.isPointWithinBounds(location)) {
            return false
        }
        
        if (gesture.state == .Ended) {
            // save location of touch
            _location = location;
            preapareDataForDrawing()
            
            // invalidate modifier context to trigger redrawing of modifier
            let context = parentSurface.renderSurface.modifierContext()
            context.invalidate()
        }
        return true
    }
    
    override func draw () {
        if (!_visible) { return }
        let parent : SCIChartSurfaceProtocol = self.parentSurface
        let surface : SCIRenderSurface = parent.renderSurface
        let context : SCIRenderContext2D = surface.modifierContext()
        
        let area : CGRect = surface.chartFrame()
        context.setDrawingArea(area) // context needs proper area for drawing (in that case it is chart area)
        
        // get coordinate calculators and calculate coordinates on screen for data point
        let data : SCIRenderPassDataProtocol = _rSeries!.currentRenderPassData
        let xCalc : SCICoordinateCalculator = data.xCoordinateCalculator()
        let yCalc : SCICoordinateCalculator = data.yCoordinateCalculator()
        let xCoord : Double = xCalc.getCoordinateFrom(_xValue)
        let yCoord : Double = yCalc.getCoordinateFrom(_yValue)
        
        // draw point marker
        _marker.drawToContext(context, atX: Float(xCoord), y: Float(yCoord))
        context.drawLineWithBrush(SCIPenSolid(colorCode: 0xFF995499, width: 0.9), fromX: Float(xCoord), y: Float(yCoord), toX: Float(xCoord), y: Float(area.height))
        // update control panel text
        let panelText = String(format: "Index: %d X:%g Y:%g", _index, _xValue, _yValue)
        _controlPanel?.setText(panelText)
    }
}

class SCSCustomModifierView: UIView {
    let sciChartView = SCSBaseChartView()
    var _controlPanel : CustomModifierControlPanel?
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        completeConfiguration()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
    }
    
    override func awakeFromNib() {
        super.awakeFromNib()
        completeConfiguration()
    }
    
    // MARK: Private Functions
    func completeConfiguration() {
        configureChartSuraface()
        addAxis()
        addDefaultModifiers()
        addSeries()
    }

    
    private func addPanel() {
        let panel = NSBundle.mainBundle().loadNibNamed("CustomModifierControlPanel",
                                                       owner: self,
                                                       options: nil)!.first
        
        if let panelValid = panel as? CustomModifierControlPanel {
            
            _controlPanel = panelValid
            
            _controlPanel?.translatesAutoresizingMaskIntoConstraints = false
            
            addSubview(_controlPanel!)
        }
    }
    
    private func addAxis() {
        let style = sciChartView.generateDefaultAxisStyle()
        sciChartView.chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: style, axisId: sciChartView.axisXId), isXAxis: true)
        sciChartView.chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: style, axisId: sciChartView.axisYId), isXAxis: false)
    }
    
    private func configureChartSuraface() {
        
        sciChartView.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(sciChartView)
        
        addPanel()
        
        let layoutDictionary = ["SciChart" : sciChartView, "Panel" : _controlPanel!]
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("V:|-(0)-[Panel(43)]-(0)-[SciChart]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[SciChart]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[Panel]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
    }
    
    private func addSeries() {
        let dataSeries = SCIXyDataSeries(XType: .Float, YType: .Float)
        
        let dataCount = 200
        for i in 0..<dataCount {
            let time = 10 * Double(i) / Double(dataCount)
            let x = time
            let y = arc4random_uniform(20);
            dataSeries.appendX(SCIGeneric(x), y: SCIGeneric(y))
        }
        dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let renderSeries = SCIFastLineRenderableSeries()
        renderSeries.dataSeries = dataSeries
        renderSeries.xAxisId = "xAxis"
        renderSeries.yAxisId = "yAxis"
        renderSeries.style.linePen = SCIPenSolid(colorCode: 0xFF99EE99, width: 0.7)
        sciChartView.chartSurface.attachRenderableSeries(renderSeries)
        
        sciChartView.chartSurface.invalidateElement()
    }
    
    func addDefaultModifiers() {
        
        let xAxisDragmodifier = SCIXAxisDragModifier()
        xAxisDragmodifier.axisId = "xAxis"
        xAxisDragmodifier.dragMode = .Scale
        xAxisDragmodifier.clipModeX = .None
        
        let yAxisDragmodifier = SCIYAxisDragModifier()
        yAxisDragmodifier.axisId = "yAxis"
        yAxisDragmodifier.dragMode = .Pan
        
        let extendZoomModifier = SCIZoomExtentsModifier()
        
        let pinchZoomModifier = SCIPinchZoomModifier()
        
        let customModifier = CustomModifier(controlPanel: _controlPanel!)
        
        let groupModifier = SCIModifierGroup(childModifiers: [xAxisDragmodifier, yAxisDragmodifier, pinchZoomModifier, extendZoomModifier, customModifier])
        
        sciChartView.chartSurface.chartModifier = groupModifier
    }
    
}
